function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { keys.push.apply(keys, Object.getOwnPropertySymbols(object)); } if (enumerableOnly) keys = keys.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import * as React from 'react';
import DownscreenContext from "./DownscreenContext.js";
import useEffectAfterMount from "./useEffectAfterMount.js";
import normalizeArrowKey from "./normalizeArrowKey.js";
import getNextIndex from "./getNextIndex.js";
import getPreviousIndex from "./getPreviousIndex.js";

var Input = function Input(_ref) {
  var children = _ref.children,
      props = _objectWithoutProperties(_ref, ["children"]);

  var _React$useContext = React.useContext(DownscreenContext),
      state = _React$useContext.state,
      setState = _React$useContext.setState,
      totalCount = _React$useContext.totalCount,
      id = _React$useContext.id,
      getMenuItemsRef = _React$useContext.getMenuItemsRef;

  var onChange = React.useCallback(function (event) {
    var inputValue = event.target.value;
    setState(function (s) {
      return _objectSpread({}, s, {
        isOpen: true,
        highlightedIndex: null,
        inputValue: inputValue
      });
    });
  }, []);
  var onBlur = React.useCallback(function () {
    setState(function (s) {
      return _objectSpread({}, s, {
        isOpen: false,
        highlightedIndex: null,
        inputValue: typeof s.lastSelectedInputValue === 'string' ? s.lastSelectedInputValue : s.inputValue
      });
    });
  }, []);
  var onKeyDown = React.useCallback(function (event) {
    var key = normalizeArrowKey(event);

    switch (key) {
      case 'Enter':
        {
          if (state.isOpen) {
            event.preventDefault();
            setState(function (s) {
              return _objectSpread({}, s, {
                isOpen: false,
                selectedIndex: s.highlightedIndex === null ? s.selectedIndex : s.highlightedIndex,
                highlightedIndex: null,
                inputValue: typeof s.lastSelectedInputValue === 'string' && s.selectedIndex === s.highlightedIndex ? s.lastSelectedInputValue : s.inputValue
              });
            });
          }

          break;
        }

      case 'Escape':
        {
          event.preventDefault();
          setState(function (s) {
            return _objectSpread({}, s, {
              isOpen: false,
              selectedIndex: null,
              highlightedIndex: null,
              inputValue: '',
              lastSelectedInputValue: null
            });
          });
          break;
        }

      case 'End':
      case 'ArrowUp':
        {
          event.preventDefault();

          if (state.isOpen) {
            setState(function (s) {
              return _objectSpread({}, s, {
                highlightedIndex: getPreviousIndex(s.highlightedIndex, totalCount, getMenuItemsRef().current)
              });
            });
          } else {
            setState(function (s) {
              return _objectSpread({}, s, {
                isOpen: true,
                lastKeyOnClose: 'ArrowUp'
              });
            });
          }

          break;
        }

      case 'Home':
      case 'ArrowDown':
        {
          event.preventDefault();

          if (state.isOpen) {
            setState(function (s) {
              return _objectSpread({}, s, {
                highlightedIndex: getNextIndex(s.highlightedIndex, totalCount, getMenuItemsRef().current)
              });
            });
          } else {
            setState(function (s) {
              return _objectSpread({}, s, {
                isOpen: true,
                lastKeyOnClose: 'ArrowDown'
              });
            });
          }

          break;
        }
    }
  }, [totalCount, state.highlightedIndex, state.isOpen]);
  var inputChildren = React.useMemo(function () {
    return children(state.selectedIndex);
  }, [state.selectedIndex]);
  useEffectAfterMount(function () {
    setState(function (s) {
      return _objectSpread({}, s, {
        inputValue: typeof inputChildren === 'string' ? inputChildren : s.inputValue,
        lastSelectedInputValue: typeof inputChildren === 'string' ? inputChildren : s.lastSelectedInputValue
      });
    });
  }, [inputChildren]);
  return React.createElement("input", _extends({
    "aria-autocomplete": "list",
    "aria-activedescendant": state.isOpen && state.highlightedIndex !== null ? "".concat(id, "-menu-item-").concat(state.highlightedIndex) : undefined,
    "aria-controls": state.isOpen ? "".concat(id, "-menu") : undefined,
    "aria-labelledby": "".concat(id, "-label"),
    autoComplete: "off",
    id: "".concat(id, "-input"),
    type: "text",
    value: state.inputValue,
    onChange: onChange,
    onBlur: onBlur,
    onKeyDown: onKeyDown
  }, props));
};

export default Input;